package dev.shadowsoffire.apotheosis.adventure.loot;

import net.minecraft.world.level.storage.loot.entries.LootPoolEntryContainer;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonElement;
import com.google.gson.reflect.TypeToken;
import com.google.gson.JsonArray;
import net.minecraft.util.GsonHelper;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonObject;
import net.minecraft.world.level.storage.loot.Serializer;
import net.minecraft.world.level.storage.loot.parameters.LootContextParams;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import net.minecraft.world.entity.player.Player;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import dev.shadowsoffire.apotheosis.adventure.compat.GameStagesCompat;
import net.minecraft.world.level.Level;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import net.minecraft.util.random.WeightedRandom;
import net.minecraft.util.random.WeightedEntry;
import java.util.function.Predicate;
import net.minecraft.world.level.storage.loot.LootContext;
import net.minecraft.world.item.ItemStack;
import java.util.function.Consumer;
import java.util.stream.Stream;
import java.util.function.Function;
import java.util.Objects;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import net.minecraft.world.level.storage.loot.functions.LootItemFunction;
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import java.util.List;
import javax.annotation.Nullable;
import net.minecraft.world.level.storage.loot.entries.LootPoolEntryType;
import net.minecraft.world.level.storage.loot.entries.LootPoolSingletonContainer;

public class GemLootPoolEntry extends LootPoolSingletonContainer
{
    public static final Serializer SERIALIZER;
    public static final LootPoolEntryType TYPE;
    @Nullable
    private final RarityClamp.Simple rarityLimit;
    private final List<DynamicHolder<Gem>> gems;
    
    public GemLootPoolEntry(@Nullable final RarityClamp.Simple rarityLimit, final List<ResourceLocation> gems, final int weight, final int quality, final LootItemCondition[] conditions, final LootItemFunction[] functions) {
        super(weight, quality, conditions, functions);
        this.rarityLimit = rarityLimit;
        final Stream<Object> stream = gems.stream();
        final GemRegistry instance = GemRegistry.INSTANCE;
        Objects.requireNonNull(instance);
        this.gems = stream.map((Function<? super Object, ?>)instance::holder).toList();
    }
    
    protected void m_6948_(final Consumer<ItemStack> list, final LootContext ctx) {
        Gem gem;
        if (!this.gems.isEmpty()) {
            final List<WeightedEntry.Wrapper<Gem>> resolved = this.gems.stream().map((Function<? super Object, ?>)this::unwrap).filter(Objects::nonNull).map(e -> e.wrap(ctx.m_78945_())).toList();
            gem = (Gem)WeightedRandom.m_216822_(ctx.m_230907_(), (List)resolved).get().m_146310_();
        }
        else {
            final Player player = findPlayer(ctx);
            if (player == null) {
                return;
            }
            gem = (Gem)GemRegistry.INSTANCE.getRandomItem(ctx.m_230907_(), ctx.m_78945_(), new Predicate[] { WeightedDynamicRegistry.IDimensional.matches((Level)ctx.m_78952_()), GameStagesCompat.IStaged.matches(player) });
        }
        final RarityClamp clamp = (this.rarityLimit == null) ? AdventureConfig.GEM_DIM_RARITIES.get(ctx.m_78952_().m_46472_().m_135782_()) : this.rarityLimit;
        final ItemStack stack = GemRegistry.createGemStack(gem, gem.clamp(LootRarity.random(ctx.m_230907_(), ctx.m_78945_(), clamp)));
        list.accept(stack);
    }
    
    public LootPoolEntryType m_6751_() {
        return GemLootPoolEntry.TYPE;
    }
    
    private Gem unwrap(final DynamicHolder<Gem> holder) {
        if (!holder.isBound()) {
            AdventureModule.LOGGER.error("A GemLootPoolEntry failed to resolve the Gem {}!", (Object)holder.getId());
            return null;
        }
        return (Gem)holder.get();
    }
    
    @Nullable
    public static Player findPlayer(final LootContext ctx) {
        final Object 78953_ = ctx.m_78953_(LootContextParams.f_81455_);
        if (78953_ instanceof final Player p) {
            return p;
        }
        final Object 78953_2 = ctx.m_78953_(LootContextParams.f_81459_);
        if (78953_2 instanceof final Player p) {
            return p;
        }
        final Object 78953_3 = ctx.m_78953_(LootContextParams.f_81458_);
        if (78953_3 instanceof final Player p) {
            return p;
        }
        if (ctx.m_78953_(LootContextParams.f_81456_) != null) {
            return (Player)ctx.m_78953_(LootContextParams.f_81456_);
        }
        return null;
    }
    
    static {
        SERIALIZER = new Serializer();
        TYPE = new LootPoolEntryType((net.minecraft.world.level.storage.loot.Serializer)GemLootPoolEntry.SERIALIZER);
    }
    
    public static class Serializer extends LootPoolSingletonContainer.Serializer<GemLootPoolEntry>
    {
        protected GemLootPoolEntry deserialize(final JsonObject obj, final JsonDeserializationContext context, final int weight, final int quality, final LootItemCondition[] lootConditions, final LootItemFunction[] lootFunctions) {
            RarityClamp.Simple rarity;
            if (obj.has("min_rarity") || obj.has("max_rarity")) {
                final DynamicHolder<LootRarity> minRarity = RarityRegistry.byLegacyId(GsonHelper.m_13906_(obj, "min_rarity"));
                final DynamicHolder<LootRarity> maxRarity = RarityRegistry.byLegacyId(GsonHelper.m_13906_(obj, "max_rarity"));
                rarity = new RarityClamp.Simple(minRarity, maxRarity);
            }
            else {
                rarity = null;
            }
            final List<String> gems = context.deserialize(GsonHelper.m_13832_(obj, "gems", new JsonArray()), new TypeToken<List<String>>() {}.getType());
            return new GemLootPoolEntry(rarity, gems.stream().map((Function<? super Object, ? extends ResourceLocation>)ResourceLocation::new).toList(), weight, quality, lootConditions, lootFunctions);
        }
        
        public void serializeCustom(final JsonObject object, final GemLootPoolEntry e, final JsonSerializationContext ctx) {
            if (e.rarityLimit != null) {
                object.addProperty("min_rarity", e.rarityLimit.min().getId().toString());
                object.addProperty("max_rarity", e.rarityLimit.max().getId().toString());
            }
            object.add("gems", ctx.serialize(e.gems));
            super.m_7219_(object, (LootPoolSingletonContainer)e, ctx);
        }
    }
}
